
#import "WindowDelegate.h"
#import "MandelView.h"
#import <stdio.h>

@implementation WindowDelegate


- setFilename:(const char *)aFilename
{
	if (filename) free(filename);
	filename = malloc(strlen(aFilename)+1);
	strcpy(filename, aFilename);
	[window setTitleWithRepresentedFilename:[NSString stringWithCString:aFilename]];
	return self;
}

- saveAs:sender
{
	const char 	*dir;
	NSString	*file;
	NSString	*tempFile;

	if (filename == 0) {
		dir = [NSHomeDirectory() cString];
		file = [window title];
		}
	else
		{
		tempFile = [NSString stringWithCString: filename];
                file = [tempFile lastPathComponent]; 
        	dir = [[tempFile stringByDeletingLastPathComponent] cString];

       		}
	
/*
#warning FactoryMethods: [SavePanel savePanel] used to be [SavePanel new].  Save panels are no longer shared.  'savePanel' returns a new, autoreleased save panel in the default configuration.  To maintain state, retain and reuse one save panel (or manually re-set the state each time.)
*/
	panel = [NSSavePanel savePanel];
	[self setFormat:formatCells];
	[panel setAccessoryView:formatBox];

/*	if ([panel runModalForDirectory:@"" file:@""]) { */

        [panel setTitle:NSLocalizedString(@"Save",@"SavePanel title")];
        if ([panel runModalForDirectory:[NSString stringWithCString:dir] 
		file:  file]) {
			[self setFilename: [[panel filename] cString]];
			return [self save:sender];
			}
	return nil;
}

- openMandFile:(NSString *) theFile
{

	NSData		*theData;
	
	theData = [NSData dataWithContentsOfFile:theFile];
	if (theData) {
		id ret = self;
		[window setTitle:NSLocalizedString(@"Loading...",@"Loading the file, title of Mandelbrot window")];
		// NS_DURING
		[theView readData:theData];
		[self setFilename:[theFile cString]];
		/*
		NS_HANDLER
			
			switch ([localException code]) {
			case NX_illegalWrite:
				NSRunAlertPanel(@"", @"Illegal write on stream", @"", nil, nil);
				ret = nil;
				break;
			case NX_illegalRead:
				NSRunAlertPanel(@"", @"Illegal read on stream", @"", nil, nil);
				ret = nil;
				break;
			case NX_illegalSeek:
				NSRunAlertPanel(@"", @"Illegal seek on stream", @"", nil, nil);
				ret = nil;
				break;
			case NX_illegalStream:
				NSRunAlertPanel(@"", @"Invalid stream", @"", nil, nil);
				ret = nil;
				break;
			case NX_streamVMError:
				NSRunAlertPanel(@"", @"VM error on stream", @"", nil, nil);
				ret = nil;
				break;
			}
		
		NS_ENDHANDLER */
		
		/*
		NXCloseMemory(theStream, NX_FREEBUFFER);
		*/

		/*
		[theData release];
		*/
		
		if (ret == nil)
			[theView cleanupBadRead];
		else
			[theView finishRead];

		return ret;
		}

	return nil;
}



- open:sender
{
	NSString	*theFile, *tempFile;
	NSArray 	*fileTypes = [NSArray arrayWithObject:@"mand"];
	NSOpenPanel	*openPanel;

	const char 	*dir;
        NSString	*file;

        if (filename == 0) {
                dir = [NSHomeDirectory() cString];
                file = [window title];
                }
        else
                {
		tempFile = [NSString stringWithCString: filename];
		file =  [tempFile lastPathComponent];
		dir = [[tempFile stringByDeletingLastPathComponent] cString];
        	}
		

	openPanel = [NSOpenPanel openPanel];
   /* 	if ([openPanel runModalForTypes:fileTypes]) { */
        [openPanel setTitle:NSLocalizedString(@"Open",@"OpenPanelTitle")];
        if ([openPanel runModalForDirectory:[NSString stringWithCString:dir]
                	file: file types:fileTypes]) {
		theFile = [openPanel filename];
		return ([self openMandFile: theFile]);
		}
	return self;
}
				

- save:sender
{

	NSMutableData	*theData;
	NSFileManager	*manager;
	NSString	*path;
	NSDictionary	*attributes;
	
	if (filename == 0) return[self saveAs: sender];
	[window setTitle:NSLocalizedString(@"Saving...","Saving the file, title of Mandelbrot window")];

/*	
	fd = open(filename, O_WRONLY|O_CREAT|O_TRUNC, 0666);
	if (fd < 0) {
		NSRunAlertPanel(@"", @"Cannot save file: %s", @"", nil, nil, strerror(errno));
		return self;
		}
*/
	manager = [NSFileManager defaultManager];
        path = [NSString stringWithCString:filename];
	if (! [manager fileExistsAtPath:path]) {
		[manager createFileAtPath:path contents:nil attributes:nil];
		attributes = [NSDictionary dictionaryWithObjectsAndKeys: 
			[NSNumber numberWithUnsignedShort: 0666], NSFilePosixPermissions, nil]; 
		[manager changeFileAttributes: attributes atPath:path];
		}
	
        theData = [NSMutableData data];
	if ([[[formatCells selectedCell] title] isEqualToString:@"MAND"])
		[theView writeMandData:theData];
	else
		[theView writeTIFFData:theData];
	[theData writeToFile:path atomically:YES];
	/*
	NXClose(theStream);
	close(fd);
	*/
	[window setTitleWithRepresentedFilename:[NSString stringWithCString:filename]];
	return self;

}

- setFormat:sender 
{
	char	*format;

	if ([[[formatCells selectedCell] title] isEqualToString:@"MAND"])
		format = "mand";
	else
		format = "tiff";
	
	/* format = NXCopyStringBuffer([[[sender selectedCell] title] cString]); */
	/* for (cc = format; *cc; cc++) 
		 *cc = NXToLower(*cc); */
/*
#warning FactoryMethods: [SavePanel savePanel] used to be [SavePanel new].  Save panels are no longer shared.  'savePanel' returns a new, autoreleased save panel in the default configuration.  To maintain state, retain and reuse one save panel (or manually re-set the state each time.)
*/
	[panel setRequiredFileType:[NSString stringWithCString:format]];
	/* free(format); */
	return self;
}	


@end
